"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.defineFactory = void 0;
const fast_element_1 = require("@microsoft/fast-element");
const defaultDefinitionOverrider = (base) => (override) => (Object.assign(Object.assign({}, base), override));
/**
 * Defines a platform custom element based on the provided type and definition.
 *
 * @example
 * An example using custom definition merge logic, useful if you need to impose hard restrictions.
 * ```ts
 * const definitionOverrider = (definition) => ({
 *    ...baseDefinition,
 *    ...definition,
 *    styles: baseDefinition.styles,
 *    shadowOptions: {
 *      mode: 'closed,
 *    },
 * });
 * export const define = defineFactory(LoginForm, definitionOverrider);
 * ```
 *
 * @example
 * You can provide the base definition if you're happy with the default spread operator merge logic.
 * ```ts
 * export const define = defineFactory(LoginForm, baseDefinition);
 * ```
 *
 * @param Type - The custom element Type / Class to define.
 * @param overriderOrBase - A function that merges definitions, or the base definition if custom merging is not needed.
 * @returns The type class.
 * @beta
 */
const defineFactory = (Type, overriderOrBase) => (definition) => {
    const overrider = typeof overriderOrBase === 'function'
        ? overriderOrBase
        : defaultDefinitionOverrider(overriderOrBase);
    return fast_element_1.FASTElement.define(class extends Type {
    }, overrider(definition));
};
exports.defineFactory = defineFactory;
